<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Models\SupportingDocs;
use App\Rules\JsonSchemaRule;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Http;
use App\Models\Order;
use DB;
use Illuminate\Contracts\Validation\Rule;
use JsonSchema\Validator as JsonSchemaValidator;

class ApiOrderController extends Controller
{
    public function store(Request $request)
    {
        $jsonSchema = storage_path("app/JSON_Schema/input_schema.json");
        $jsonData = $request->json()->all();

        $jsonData = json_decode(json_encode($jsonData));

        $validator = new JsonSchemaValidator();
        $validator->validate($jsonData, (object)['$ref' => 'file://' . $jsonSchema]);

        if ($validator->isValid()) {
            $client_id = $request->input('OrderInfo.client_id');
            $service_id = $request->input('OrderInfo.service_id');
            $client_order_id = $request->input('OrderInfo.client_order_id');

            if(!empty($request->input('PropertyData.legal_document'))) {
                $decodedData = base64_decode($request->input('PropertyData.legal_document'));
                $filename = uniqid('legal_document_') . '.' . 'pdf';
                \Storage::put('Legal_Document_Inputs/' . $filename, $decodedData);
                $jsonData->PropertyData->legal_document = $filename;
            }

            $orderId = Order::insertGetId([
                'input' => json_encode($jsonData),
                'client_id' => $client_id,
                'service_id' => $service_id,
                'client_order_id' => $client_order_id,
                'is_api_order' => 1,
                'status' => 1
            ]);
            $OrderData = Order::leftJoin('status', 'status.id', '=', 'orders.status')
                        ->leftJoin('clients', 'clients.id', '=', 'orders.client_id')
                        ->leftJoin('services', 'services.id', '=', 'orders.service_id')
                        ->select(
                            'orders.id as OrderId',
                            'status.status as OrderStatus',
                            'clients.client_name as Client',
                            'services.service_name as Service',
                            'orders.created_at as OrderCreatedAt'
                        )->where('orders.id', $orderId)->first();

            return response()->json($OrderData);
        } else {
            $errors = $validator->getErrors();
            return response()->json(['error' => $errors], 422);
        }
    }

    public function OrderHandover($orderId, $statusId, $comments)
    {
        $endpoint_url = 'http://147.135.95.203:3489/api';

        try
        {
            DB::table('api_tokens')->where('expires_at', '<', now()->addMinutes(60))->delete();
            DB::table('oauth_access_tokens')->where('expires_at', '<', now()->addMinutes(60))->delete();
            $tokenData = DB::table('api_tokens')->where('user_id', 1)->where('client_id', 1)->where('expires_at', '>', now()->addMinutes(1))->first();
            if(empty($tokenData)) {

                $date = now()->addHour(12);
                $response = Http::timeout(60)
                                ->post($endpoint_url. '/login', [
                                    'username' => 'stellar',
                                    'password' => 'uatstellar2@',
                                ]);
                $responseData = json_decode($response->body(), true);
                DB::table('api_tokens')->insert(['user_id' => 1, 'client_id' => 1, 'access_token' => $responseData['token'], 'expires_at' => $date->format('Y-m-d H:i:s')]);
                $accessToken = $responseData['token'];
            } else {
                $accessToken = $tokenData->access_token;
            }

            $order = Order::where('id', $orderId)->first();

            if (!empty($order)) {
                $serviceFolder = $order->service_id == 1 ? 'SFHDF_Form_Output' : 'LV_Form_Output';
                $filePath = storage_path("app/$serviceFolder/{$order->id}");

                $fileNames = ['Merged_Certificate.pdf', 'Certificate.pdf'];

                foreach ($fileNames as $fileName) {
                    if (File::exists("$filePath/$fileName")) {
                        $fileContent = base64_encode(file_get_contents("$filePath/$fileName"));
                        break;
                    }
                }
            }

            if($statusId == 8 || $statusId == 9) {
                $response = Http::withHeaders([
                    'Authorization' => 'Bearer ' . $accessToken,
                ])->post($endpoint_url.'/status', [
                    'client_order_id' => $order->client_order_id,
                    'status_id' => $statusId,
                    'comments' => $comments,
                    'file' => $fileContent,
                    'json_response' => $order->output,
                ], ['Content-Type' => 'application/json']);
            } else {
                $response = Http::withHeaders([
                    'Authorization' => 'Bearer ' . $accessToken,
                ])->post($endpoint_url.'/status', [
                    'client_order_id' => $order->client_order_id,
                    'status_id' => $statusId,
                    'comments' => $comments,
                ], ['Content-Type' => 'application/json']);
            }

            return $response;
        } catch(Exception $e) {
            $responseDt['ResponseStatus'] = [
                "IsSuccess" => false,
                "StatusMessage" => "API Credentials are no longer valid",
            ];
            return response()->json($responseEnDt);
        }

        return response()->json(["result" => $responseData ?? $errorResponse]);
    }

    public function clientupdate(Request $request)
    {
        $jsonData = $request->json()->all();
        $filename = null;

        if (!empty($jsonData['order_id'])) {
            $Order = DB::table('orders')->where('id', $jsonData['order_id'])->first();

            if (!empty($request->input('file'))) {
                $decodedData = base64_decode($request->input('file'));
                $filename = uniqid() . '.pdf';

                // Assuming $OrderData is defined somewhere in your code
                if ($Order) {
                    $basepath = $Order->service_id == 1 ? 'SFHDF_Form_Output' : 'LV_Form_Output';
                    \Storage::put("$basepath/{$Order->id}/$filename", $decodedData);

                    SupportingDocs::insertGetId([
                        'order_id' => $Order->id,
                        'pdf_file' => $filename,
                        'created_at' => now(),
                    ]);
                }
            }
        }

        $latestRecord = DB::table('orders_status_history')
        ->where('order_id', $jsonData['order_id'])
        ->where('status_id', $jsonData['status_id'])
        ->latest('updated_at')
        ->first();

        // Update the latest matching record
        if ($latestRecord) {
            DB::table('orders_status_history')
                ->where('id', $latestRecord->id) // Assuming there's an 'id' column
                ->update(['client_comments' => $jsonData['comments'], 'attachment' => $filename ?? null]);

            DB::table('orders')
                 ->where('id', $latestRecord->order_id)
                 ->update(['status' => 6]);

            DB::table('orders_status_history')
                ->insert(['order_id' => $latestRecord->order_id ,'status_id' => 6, 'comments' => "Received Clarification from Client"]);
        }

        return response()->json(['OrderId' => $jsonData['order_id'],'message' => "Inputs added successfully"]);
    }
}
