<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\Order;
use App\Models\User;
use App\Models\Service;
use Session;
use DB;

class AssignOrderController extends Controller
{
     /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */

    public function ordersAssignment(Request $request)
    {
        if(Session::get('user_type_id') == 1) {
            // $users = User::where('is_active', 1)->select('id', 'first_name', 'last_name')->get();
            $users = User::where('users.is_active', 1)
            ->leftJoin('user_service_mapping', 'users.id', '=', 'user_service_mapping.user_id')
            ->leftJoin('services', 'user_service_mapping.service_id', '=', 'services.id')
            ->where('services.is_active', 1)
            ->selectRaw('users.id as user_id, users.first_name, users.last_name, GROUP_CONCAT(services.id) as service_ids')
            ->groupBy('users.id', 'users.first_name', 'users.last_name')
            ->get();

        } else {
            $users = User::where('users.is_active', 1)
                ->leftjoin('user_service_mapping', 'users.id', '=', 'user_service_mapping.user_id')
                ->leftjoin('services', 'user_service_mapping.service_id', '=', 'services.id')
                ->where('services.is_active', 1)
                ->select('users.id as user_id', 'users.first_name', 'users.last_name', 'services.id as service_id', 'services.company_id', 'services.service_name', 'services.short_code', 'services.description', 'services.is_active')
                ->where('users.company_id', Session::get('company_id'))
                ->get();

        }

        if ($request->is('ordersAssignment/assign_orders')) {
            $orders = Order::with('statusTypes', 'clients', 'services', 'assignedUsers', 'updatedByUsers')->whereIn('status', [1])->whereNull('assigned_user_id')->get();
            $serviceList = Service::select('id','service_name')->get();
            return view('app.assignorders.assign', compact('orders', 'users', 'serviceList'));
        } else if($request->is('ordersAssignment/assigned_orders')){
            $orders = Order::with('statusTypes', 'clients', 'services', 'assignedUsers', 'updatedByUsers')->whereNotIn('status', [8,9,10])->whereNotNull('assigned_user_id')->get();
            return view('app.assignorders.assigned', compact('orders'));
        } else if($request->is('ordersAssignment/unassigned_orders')){
            $orders = Order::with('statusTypes', 'clients', 'services', 'assignedUsers', 'updatedByUsers')->whereNotIn('status', [1])->whereNull('assigned_user_id')->get();
            return view('app.assignorders.unassigned', compact('orders', 'users'));
        }
        $orders = Order::with('statusTypes', 'clients', 'services', 'assignedUsers', 'updatedByUsers')->whereIn('status', [1])->whereNull('assigned_user_id')->get();
        return view('app.assignorders.assign', compact('orders', 'users'));
    }

    public function assignment_update(Request $request)
    {
        $input = $request->all();
        $type = $input['type'];

        $validatedData = $request->validate([
            'orders' => 'required',
        ]);

        if(count($request->input('orders')) > 0) {
            $orderIds = $input['orders'];
            if($type == 1) {
                foreach($orderIds as $orderId) {
                    Order::where('id', $orderId)->update(['assigned_user_id' => $input['user_id'], 'status' => 2]);
                }
                return redirect()->route('assign_orders');
            } elseif ($type == 2) {
                foreach($orderIds as $orderId) {
                    Order::where('id', $orderId)->update(['assigned_user_id' => NULL]);
                }
                return redirect()->route('assigned_orders');
            } elseif ($type == 3){
                foreach($orderIds as $orderId) {
                    Order::where('id', $orderId)->update(['assigned_user_id' => $input['user_id']]);
                }
                return redirect()->route('unassigned_orders');
            }
        }
    }

    public function getServiceUsers(Request $request){

        $serviceIds = array_unique($request->input('serviceId'));

        $users = DB::table('users')
            ->join('user_service_mapping', 'users.id', '=', 'user_service_mapping.user_id')
            ->whereIn('user_service_mapping.service_id', $serviceIds)
            ->select('users.id', 'users.first_name', 'users.last_name')
            ->groupBy('users.id', 'users.company_id', 'users.first_name', 'users.last_name', 'users.email', 'users.password', 'users.user_type_id', 'users.contact_no', 'users.is_active', 'users.remember_token')
            ->havingRaw('COUNT(DISTINCT user_service_mapping.service_id) = ' . count($serviceIds))
            ->get();



        return response()->json($users, 200);

    }
}
