<?php

namespace App\Http\Controllers;

use PDF;
use App\Models\Order;
use App\Models\SupportingDocs;
use mikehaertl\pdftk\Pdf as pdfTk;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;

class FloodCertPDFController extends Controller
{
    public function fillPDF($orderId) {
        $templatePdf = public_path('SFHDF_Form_2023/fema_form-ff-206-fy-21-116_112022.pdf');
        $directoryPath = "SFHDF_Form_Output/{$orderId}";
        $outputFilePath = storage_path("app/{$directoryPath}/Certificate.pdf");

        if (!Storage::exists($directoryPath)) {
            Storage::makeDirectory($directoryPath, 0777, true);
        } else {
            chmod(storage_path("app/{$directoryPath}"), 0777);
        }

        $pdf = new pdfTk($templatePdf);
        $orderData = Order::where('id', $orderId)->first();
        $data = json_decode($orderData->output, true);
        // Combine multiple fields into a single field for PREPARER'S INFORMATION
        $preparerName = $data['flood_cert_data']['preparer_information']['name'];
        $preparerAddress = $data['flood_cert_data']['preparer_information']['address'];
        $preparerNumber = $data['flood_cert_data']['preparer_information']['number'];

        $lenderName = $data['flood_cert_data']['loan_information']['lender_name'];
        $lenderAddress = $data['flood_cert_data']['loan_information']['lender_address'];
        $lenderInformation = "$lenderName\n$lenderAddress";
        $preparerInformation = "$preparerName\n$preparerAddress\nNumber: $preparerNumber";


        // Map data to PDF fields
        $pdfFields = [
            'form1[0].#subform[3].TextField1[0]' => $lenderInformation,
            'form1[0].#subform[3].TextField2[0]' => $data['flood_cert_data']['loan_information']['collateral_desc'],
            'form1[0].#subform[3].TextField3[0]' => $data['flood_cert_data']['loan_information']['loan_identifier'],
            'form1[0].#subform[3].TextField16[0]' => $data['flood_cert_data']['loan_information']['lender_id'],
            'form1[0].#subform[3].TextField4[0]' => $data['flood_cert_data']['loan_information']['amount_of_flood_insurance'],
            'form1[0].#subform[3].TextField5[0]' => $data['flood_cert_data']['jurisdiction']['community_name'],
            'form1[0].#subform[3].TextField6[0]' => $data['flood_cert_data']['jurisdiction']['county'],
            'form1[0].#subform[3].TextField15[0]' => $data['flood_cert_data']['jurisdiction']['state'],
            'form1[0].#subform[3].NumericField2[0]' => $data['flood_cert_data']['jurisdiction']['community_num'],
            'form1[0].#subform[3].TextField8[0]' => $data['flood_cert_data']['flood_insurance']['map_num'],
            'form1[0].#subform[3].TextField9[0]' => $data['flood_cert_data']['flood_insurance']['map_panel'],
            'form1[0].#subform[3].RadioButtonList[0]' => $data['flood_cert_data']['flood_insurance']['map_change'],
            'form1[0].#subform[3].DateField1[0]' => $data['flood_cert_data']['flood_insurance']['date'],
            'form1[0].#subform[3].TextField10[0]' => $data['flood_cert_data']['flood_insurance']['flood_zone'],
            'form1[0].#subform[3].TextField11[0]' => $data['flood_cert_data']['flood_insurance']['map'],
            'form1[0].#subform[3].TextField12[0]' => $data['flood_cert_data']['flood_insurance']['case_no'],
            'form1[0].#subform[3].CheckBox3[0]' => $data['flood_cert_data']['ffi_availability']['insurance_available'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox4[0]' => $data['flood_cert_data']['ffi_availability']['regular_program'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox5[0]' => $data['flood_cert_data']['ffi_availability']['emergency_program'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox6[0]' => $data['flood_cert_data']['ffi_availability']['insurance_not_available'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox7[0]' => $data['flood_cert_data']['ffi_availability']['federal_flood_insurance'] ? '1' : 'Off',
            'form1[0].#subform[3].DateField3[0]' => $data['flood_cert_data']['ffi_availability']['designation_date'],
            'form1[0].#subform[3].CheckBox8[0]' => ($data['flood_cert_data']['determination']['in_special_flood_hazard_area'] == "1") ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox9[0]' => ($data['flood_cert_data']['determination']['in_special_flood_hazard_area'] == "off") ? '1' : 'Off',
            'form1[0].#subform[3].TextField13[0]' => $preparerInformation,
            'form1[0].#subform[3].TextField14[0]' => $data['flood_cert_data']['comments']['comments'],
            'form1[0].#subform[3].DateField2[0]' => $data['flood_cert_data']['preparer_information']['date_of_determination'],
        ];

        // $result = $pdf->fillForm($pdfFields)->needAppearances()->saveAs($outputFilePath);
        $result = $pdf->fillForm($pdfFields)->flatten()->saveAs($outputFilePath);

        $msg = '';

        if ($result === false)
        {
            $msg = $pdf->getError();
        } else {
            $msg = "Form Filled Successfully";
        }

        if(substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($directoryPath, 0777);
        }

        return response()->json($msg);
    }

    protected function changeFolderPermissions($path, $permissions)
    {
        if (file_exists($path) || is_dir($path)) {
            chmod($path, $permissions);
            $items = scandir($path);

            foreach ($items as $item) {
                if ($item != '.' && $item != '..') {
                    $itemPath = $path . '/' . $item;
                    if (is_dir($itemPath)) {
                        $this->changeFolderPermissions($itemPath, $permissions);
                    } else {
                        chmod($itemPath, $permissions);
                    }
                }
            }
        }
    }
}
