<?php

namespace App\Http\Controllers;

use DB;
use Hash;
use Session;
use App\Models\User;
use App\Models\Service;
use App\Models\Order;
use App\Models\Company;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Http\Response;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index(Request $request)
    {
        $user = User::where('id', Auth::id())->first();

        if ($user->is_active == 1) {
            session(['uid' => Auth::id()]);
            session(['user_type_id' => $user->user_type_id]);
            session(['company_id' => isset($user->company_id) ? $user->company_id : 0]);
        } else {
            Auth::logout();

            return redirect('/');
        }
        return view('app.dashboard.index');
    }

    public function profileupdate(Request $request)
    {
        $request->validate([
            'new_password' => 'required|min:8|confirmed',
        ]);

        $user = Auth::user();
        $user->password = Hash::make($request->input('new_password'));
        $user->save();

        return redirect()->route('home')->with('success', 'Password updated successfully.');
    }

    public function profileEdit(Request $request)
    {
        return view('app.dashboard.profile_edit');
    }

    public function apitoken(Request $request)
    {
        $user = Auth::user();
        DB::table('personal_access_tokens')->where('tokenable_id', Auth::id())->whereNull('expires_at')->update(['expires_at' => now()]);
        $token = substr($user->createToken('authToken')->plainTextToken,2);
        return view('app.dashboard.profile_edit', compact('token'));
    }

    public function showPdf($folder, $filename)
    {
        $folder = str_ireplace('@@', '/', $folder);
        $filePath = "{$folder}/{$filename}";

        if (!Storage::exists($filePath)) {
            abort(404);
        }

        $content = Storage::get($filePath);

        return (new Response($content, 200))
            ->header('Content-Type', 'application/pdf');
    }
}

