<?php

namespace App\Http\Controllers;

use DB;
use Hash;
use Session;
use App\Models\User;
use App\Models\Status;
use App\Models\Service;
use App\Models\Order;
use App\Models\OrderStatusHistory;
use App\Models\SupportingDocs;
use App\Models\Company;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\Api\ApiOrderController;

class OrderController extends Controller
{
    protected $floodCertPDFController;
    protected $pdfController;
    protected $apiOrderController;


    public function __construct(FloodCertPDFController $floodCertPDFController, PDFController $pdfController, ApiOrderController $apiOrderController)
    {
        $this->floodCertPDFController = $floodCertPDFController;
        $this->pdfController = $pdfController;
        $this->apiOrderController = $apiOrderController;
    }

    public function orders(Request $request)
    {
        $orderData = DB::table('orders')->leftJoin('services', 'orders.service_id', '=', 'services.id')->get();

        $status_id = Status::pluck('id')->toArray();

        return view('app.orders.orderlist', compact('orderData', 'status_id'));
    }

    public function getOrderData(Request $request)
    {
        $status = $request->status;
        $user = Auth::user();
        $company_id = Session::get('company_id');
        $userType = $user->role;
        $mappedServices = DB::table('user_service_mapping')
                            ->where('user_id', Auth::id())
                            ->where('is_active', 1)
                            ->pluck('service_id')
                            ->toArray();
        $orders = Order::with([
                    'statusTypes',
                    'assignedUsers',
                    'services',
                    'clients',
                    'createdBy',
                    'order_status_history'
                ])
                ->when($user->hasRole('Super Admin'), function ($query) use ($status) {
                    return $query->whereHas('statusTypes', function ($subQuery) use ($status) {
                        $subQuery->when($status !== 'All', function ($condition) use ($status) {
                            return $condition->where('is_active', 1)->where('id', $status);
                        }, function ($condition) {
                            return $condition->where('is_active', 1);
                        });
                    });
                })
                ->when($user->hasRole('Admin') || $user->hasRole('Lead'), function ($query) use ($status, $company_id) {
                    return $query
                        ->whereHas('services', function ($subQuery) use ($company_id) {
                            $subQuery->where('company_id', $company_id)->where('is_active', 1);
                        })
                        ->whereHas('statusTypes', function ($subQuery) use ($status) {
                            $subQuery->when($status !== 'All', function ($condition) use ($status) {
                                return $condition->where('is_active', 1)->where('id', $status);
                            }, function ($condition) {
                                return $condition->where('is_active', 1);
                            });
                        });
                })
                ->when($user->hasRole('User'), function ($query) use ($status, $company_id, $mappedServices) {
                    return $query
                        ->when($status != 1, function ($subQuery) {
                            return $subQuery->where('assigned_user_id', Auth::id());
                        })
                        ->whereHas('services', function ($subQuery) use ($company_id) {
                            $subQuery->where('company_id', $company_id)->where('is_active', 1);
                        })
                        ->whereHas('statusTypes', function ($subQuery) use ($status) {
                            $subQuery->when($status !== 'All', function ($condition) use ($status) {
                                return $condition->where('is_active', 1)->where('id', $status);
                            }, function ($condition) {
                                return $condition->where('is_active', 1);
                            });
                        })->whereIn('service_id', $mappedServices);
                })
                ->get();

                $statusCounts = $this->getStatusCountsBasedOnUserRole($user);

                return response()->json(['OrderData' => $orders, 'StatusCounts' => $statusCounts,  'UserType' => $userType]);
            }

    public function getStatusCountsBasedOnUserRole($user)
    {
        $statusCountsQuery = Order::query();

        if ($user->hasRole('Super Admin')) {
            $statusCounts = $statusCountsQuery->groupBy('status')->selectRaw('count(*) as count, status')->pluck('count', 'status');
        } elseif ($user->hasRole('Admin') || $user->hasRole('Lead')) {
            $statusCounts1 = Order::where('status', 1)->count();
            $statusCounts = $statusCountsQuery->whereHas('statusTypes', function ($subQuery) {
                $subQuery->where('is_active', 1);
            })->where('assigned_user_id', Auth::id())->groupBy('status')->selectRaw('count(*) as count, status')->pluck('count', 'status');

            $statusCounts[1] = $statusCounts1;
        } elseif ($user->hasRole('User')) {
            $statusCounts1 = Order::where('status', 1)->count();
            $statusCounts = $statusCountsQuery->when(DB::raw('status <> 1'), function ($subQuery) {
                return $subQuery->where('assigned_user_id', Auth::id());
            })->groupBy('status')->selectRaw('count(*) as count, status')->pluck('count', 'status');
            $statusCounts[1] = $statusCounts1;
        }

        return $statusCounts;
    }

    public function orderform(Request $request, $order_id){
        $order = Order::findOrFail($order_id);

            $jsonString = json_decode(stripslashes($order->input), true);
            // Check if decoding was successful
            if ($jsonString !== null) {
                // Convert the associative array back to a JSON string
                $formattedJson = json_encode($jsonString, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                // Remove unwanted tags
                $input_json = strip_tags(html_entity_decode($formattedJson));
            } else {
                // Handle JSON decoding error
                $input_json = 'Invalid JSON';
            }

        $orderStatus = Status::all();
        $services = Service::all();
        $status_type = Status::where('id', $order->status)->value('status');
        $service_name = Service::where('id', $order->service_id)->value('service_name');

        $orderData = [
            'status_type' => $status_type,
            'service_name' => $service_name,
        ];

        if($order->status == 2) {
            Order::where('id', $order_id)->update(['status' => 3]);
            DB::table('orders_status_history')
            ->insert(['order_id' => $order_id ,'status_id' => 3, 'added_by' => Auth::id(), 'updated_at' => now()]);
            $comments = str_ireplace("\n","<br>", "Order is In-Progress");
            $response = $this->apiOrderController->OrderHandover($order_id, 3, $comments);
        }

        $status_change = DB::table('status')->whereNotIn('id', [1,2,4,5,6,10,11,12])->get();

        $statusHistory = DB::table('orders_status_history')
                        ->leftjoin('orders', 'orders.id', '=', 'orders_status_history.order_id')
                        ->leftjoin('users', 'users.id', '=', 'orders_status_history.added_by')
                        ->leftjoin('status', 'status.id', '=', 'orders_status_history.status_id')
                        ->select('orders_status_history.*', 'users.first_name', 'users.last_name', 'status.status')
                        ->where('orders.id', $order_id)
                        ->where('orders.is_active', 1)
                        ->whereIN('orders_status_history.status_id', [4,5,6,10,11,12])
                        ->orderBy('orders_status_history.updated_at', 'asc')
                        ->get();

        $supportingdocs = SupportingDocs::where('order_id',$order_id)->get();

        if($order->service_id == 1){
            return view('app.certforms.floodCertForm', compact('order', 'orderStatus','supportingdocs', 'orderData', 'statusHistory', 'status_change', 'input_json'));
        }
        if($order->service_id == 2){
            return view('app.certforms.legalVestingForm', compact('order', 'orderStatus','supportingdocs', 'orderData', 'statusHistory', 'status_change', 'input_json'));
        }
    }

    public function  order_status_history(Request $request)
    {
        $request->validate([
            'orderid'=> 'required',
        ]);

        $order_history = DB::table('orders_status_history')
                        ->leftjoin('status', 'orders_status_history.status_id', '=', 'status.id')
                        ->leftjoin('users', 'orders_status_history.added_by', '=', 'users.id')
                        ->select('status.status', 'users.first_name', 'users.last_name', 'orders_status_history.comments', 'orders_status_history.updated_at')
                        ->where('order_id', $request->orderid)->get();

        return response()->json($order_history);
    }

    public function  other_status_update(Request $request)
    {
        $request->validate([
            'orderid'=> 'required'
        ]);

        $orderData = DB::table('orders')
                        ->where('id','=',  $request->orderid)->update(['status' => $request->other_status]);
        if($request->other_status == 4) {
            DB::table('orders_status_history')
                        ->insert(['order_id' => $request->orderid ,'status_id' => $request->other_status, 'added_by' => Auth::id(), 'followup_date' => $request->followup_date, 'eta_date' => $request->estimated_date, 'comments' => str_ireplace("\n","<br>",$request->other_status_comments), 'updated_at' => now()]);
        }
        if($request->other_status == 10 || $request->other_status == 11 || $request->other_status == 12 || $request->other_status == 5 || $request->other_status == 6) {
            DB::table('orders_status_history')
            ->insert(['order_id' => $request->orderid ,'status_id' => $request->other_status, 'added_by' => Auth::id(), 'comments' => str_ireplace("\n","<br>",$request->other_status_comments), 'updated_at' => now()]);
        }

        $comments = str_ireplace("\n","<br>",$request->other_status_comments);
        $response = $this->apiOrderController->OrderHandover($request->orderid, $request->other_status, $comments);

        return $response;
    }

    public function order_status_change (Request $request) {
        Order::where('id', $request->orderid)->update(['status' => $request->status]);
        DB::table('orders_status_history')
            ->insert(['order_id' => $request->orderid ,'status_id' => $request->status, 'added_by' => Auth::id(), 'updated_at' => now()]);

        $response = '';
        if($request->status == 8 || $request->status == 9) {
            $comments = "Order Completed successfully";
            $response = $this->apiOrderController->OrderHandover($request->orderid, $request->status, $comments);
        } else {
            $comments = "";
            $response = $this->apiOrderController->OrderHandover($request->orderid, $request->status, $comments);
        }

        return $response;
    }

    public function submitOrderData(Request $request)
    {
        $order_id = $request->order_id;

        $data = json_decode($request->json_data, true);

        $loan_information = [
            'lender_name' => $data['lender_name'] ?? Null,
            'lender_address' => $data['lender_address'] ?? Null,
            'collateral_desc' => $data['collateral_desc'] ?? Null,
            'lender_id' => $data['lender_id'] ?? Null,
            'loan_identifier' => $data['loan_identifier'] ?? Null,
            'amount_of_flood_insurance' => $data['amount_of_flood_insurance'] ?? Null,
        ];

        $jurisdiction = [
            'community_name' => $data['community_name'] ?? Null,
            'county' => $data['nfip_county'] ?? Null,
            'state' => $data['nfip_state'] ?? Null,
            'community_num' => $data['community_num'] ?? Null,
        ];

        $flood_insurance = [
            'map_num' => $data['map_num'] ?? Null,
            'map_panel' => $data['map_panel'] ?? Null,
            'map_change' => $data['map_change'] ?? Null,
            'date' => $data['date'] ?? Null,
            'case_no' => $data['case_no'] ?? Null,
            'flood_zone' => $data['flood_zone'] ?? Null,
            'map' => $data['nfip_map'] ?? Null,
        ];

        $ffi_availability = [
            'insurance_available' => $data['insurance_available'] ?? Null,
            'regular_program' => $data['regular_program'] ?? Null,
            'emergency_program' => $data['emergency_program'] ?? Null,
            'insurance_not_available' => $data['insurance_not_available'] ?? Null,
            'federal_flood_insurance' => $data['federal_flood_insurance'] ?? Null,
            'designation_date' => $data['designation_date'] ?? Null,
        ];

        $determination = [
            'in_special_flood_hazard_area' => $data['in_special_flood_hazard_area'] ?? Null,
        ];

        $comments = [
            'comments' => $data['comments'] ?? Null,
        ];

        $preparer_information = [
            'name' => $data['name'] ?? Null,
            'address' => $data['preparer_address'] ?? Null,
            'number' => $data['number'] ?? Null,
            'date_of_determination' => $data['date_of_determination'] ?? Null,
        ];


        $all_flood_cert_data = [
            'loan_information' => $loan_information,
            'jurisdiction' => $jurisdiction,
            'flood_insurance' => $flood_insurance,
            'ffi_availability' => $ffi_availability,
            'determination' => $determination,
            'comments' => $comments,
            'preparer_information' => $preparer_information,
        ];

        $flood_cert_data = [
            'flood_cert_data' => $all_flood_cert_data
        ];

        // return response()->json($flood_cert_data, 200);

        $responseDataJson = json_encode($flood_cert_data, JSON_UNESCAPED_UNICODE);

        DB::table('orders')->where('id', $order_id)->update(['output' => $responseDataJson]);

        $fill_result = $this->floodCertPDFController->fillPDF($order_id);

        return response()->json(['success' => true]);
    }


    public function upload_supporting_doc(Request $request)
    {

        $result = $this->pdfController->upload_supporting_doc($request->orderid, $request->refdocs, $request->baseDirPath);

        return response()->json($result);
    }

    public function delete_supporting_doc(Request $request)
    {
        $result = $this->pdfController->delete_supporting_doc($request->orderid, $request->baseDirPath, $request->supportid);

        return response()->json($result);
    }

    public function open_supporting_doc(Request $request)
    {
        $supportingdocs = SupportingDocs::find($request->docid);

        if (!$supportingdocs) {
            return response()->json(['error' => 'Supporting document not found.'], 404);
        }

        $filePath = storage_path('app' . '/' . $request->baseDirPath . '/' . $request->orderid . '/' . $supportingdocs->pdf_file);

        if (!file_exists($filePath)) {
            return response()->json(['error' => 'File not found.'], 404);
        }

        return response()->file($filePath, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="' . $supportingdocs->pdf_file . '"',
        ]);
    }

    public function submitLegalVestingData(Request $request)
    {
        $order_id = $request->order_id;

        $orderData = Order::where('id', $order_id)->first();

        $data = json_decode($request->json_data, true);

        $OrderInformation = [
            'file_order_no' => $data['file_order_no'] ?? null,
            'product_name' => $data['product_name'] ?? null,
            'borrower_names' => $data['borrower_names'] ?? null,
            'property_address' => $data['orderinfo_property_address'] ?? null,
            'city' => $data['orderinfo_city'] ?? null,
            'state' => $data['orderinfo_state'] ?? null,
            'county' => $data['orderinfo_county'] ?? null,
            'city_state_county' => trim(trim(trim(($data['orderinfo_city'] ?? '') . ', ' . ($data['orderinfo_state'] ?? '') . ', ' . ($data['orderinfo_county'] ?? '')),",")),
        ];

        $SearchInformation = [
            'search_date' => $data['search_date'] ?? null,
            'effective_date' => $data['effective_date'] ?? null,
            'names_searched' => $data['names_searched'] ?? null,
            'address_parcel_searched' => $data['address_parcel'] ?? null,
        ];
        $AssessmentInformation = [
            'comments' => $data['assessment_info_data'] ?? null,
        ];
        $CurrentOwnerVesting = [
            'current_ov' => $data['current_ov'] ?? null,
            'comments' => $data['comment_ov'] ?? null,
        ];
        $VestingDeed = [
            'deed_type' => $data['deed_type'] ?? null,
            'dated_date' => $data['dated_date'] ?? null,
            'book_page' => $data['book_page'] ?? null,
            'instrument_no' => $data['instrument_no'] ?? null,
            'comments' => $data['comment_vd'] ?? null,
            'grantor' => $data['grantor'] ?? null,
            'grantee' => $data['grantee'] ?? null,
            'recorded_date' => $data['recorded_date'] ?? null,
        ];
        $ForPreamble = [
            'city_township_parish' => $data['city_township_parish'] ?? null,
        ];
        $AdditionalNotes =  $data['additional_notes_data'] ?? null;
        $LegalDescription =  $data['legal_desc_data'] ?? null;

        $LV_Data = [
            'OrderInformation' => $OrderInformation,
            'SearchInformation' => $SearchInformation,
            'AssessmentInformation' => $AssessmentInformation,
            'CurrentOwnerVesting' => $CurrentOwnerVesting,
            'VestingDeed' => $VestingDeed,
            'ForPreamble' => $ForPreamble,
            'AdditionalNotes' => $AdditionalNotes,
            'LegalDescription' => $LegalDescription,
        ];

        function recursiveUppercase($array) {
            $result = [];
            foreach ($array as $key => $value) {
                if (is_array($value)) {
                    $result[$key] = recursiveUppercase($value);
                } else {
                    $result[$key] = is_string($value) ? strtoupper($value) : $value;
                }
            }
            return $result;
        }

        $LV_Data = recursiveUppercase($LV_Data);

        $legal_vesting_data = [
            'legal_vesting_data' => $LV_Data
        ];

        $responseDataJson = json_encode($legal_vesting_data, JSON_UNESCAPED_UNICODE);

        DB::table('orders')->where('id', $order_id)->update(['output' => $responseDataJson]);

        $clientData = DB::table('clients')->where("id", $orderData->client_id)->first();

        $pdf = \PDF::loadView('pdf.legal_vesting_template', ['record' => $LV_Data, 'clientData' => $clientData]);
        $directoryPath = "LV_Form_Output/{$order_id}";
        $outputFilePath = storage_path("app/{$directoryPath}/Certificate.pdf");

        if (!Storage::exists($directoryPath)) {
            Storage::makeDirectory($directoryPath, 0777, true);
        } else {
            chmod(storage_path("app/{$directoryPath}"), 0777);
        }

        $result = $pdf->save($outputFilePath);

        if(substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($directoryPath, 0777);
        }

        if ($result === false)
        {
            $msg = $pdf->getError();
        } else {
            $msg = "PDF Created Successfully";
        }

        return response()->json($msg);
    }

    protected function changeFolderPermissions($path, $permissions)
    {
        if (file_exists($path) || is_dir($path)) {
            chmod($path, $permissions);
            $items = scandir($path);

            foreach ($items as $item) {
                if ($item != '.' && $item != '..') {
                    $itemPath = $path . '/' . $item;
                    if (is_dir($itemPath)) {
                        $this->changeFolderPermissions($itemPath, $permissions);
                    } else {
                        chmod($itemPath, $permissions);
                    }
                }
            }
        }
    }

    public function assignOrder(Request $request)
    {
        $orderId = $request->input('orderId');

        $order = Order::find($orderId);

        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }

        static $isAssigning = false;

        if ($isAssigning) {
            return response()->json(['message' => 'Order is already being processed']);
        }

        $isAssigning = true;

        try {

            if ($order->status == 1) {
                $order->update([
                    'status' => 3,
                    'assigned_user_id' => Auth::id(),
                ]);

                $comments = str_ireplace("\n","<br>", "Order is In-Progress");
                $response = $this->apiOrderController->OrderHandover($orderId, 3, $comments);

                return response()->json(['message' => 'Order assigned successfully']);
            } else {
                return response()->json(['message' => 'Order already assigned']);
            }
        } catch (\Exception $e) {

            $isAssigning = false;

            return response()->json(['error' => 'Error assigning order']);
        } finally {
            $isAssigning = false;
        }
    }

    public function getAttachments(Request $request) {
        $order_id = $request->order_id;
        $updates = DB::table('orders_status_history')
                    ->leftjoin('orders', 'orders.id', '=', 'orders_status_history.order_id')
                    ->leftjoin('users', 'users.id', '=', 'orders_status_history.added_by')
                    ->leftjoin('status', 'status.id', '=', 'orders_status_history.status_id')
                    ->select('orders_status_history.*', 'users.first_name', 'users.last_name', 'status.status')
                    ->where('orders.id', $order_id)
                    ->where('orders.is_active', 1)
                    ->whereIN('orders_status_history.status_id', [4,5,6,10,11,12])
                    ->orderBy('orders_status_history.updated_at', 'asc')
                    ->get();
        return response()->json(['OtherStatusData' => $updates ?? []]);
    }

    public function open_legal_doc(Request $request)
    {
        $order = Order::findOrFail($request->orderid);

        if(!empty($order->input['PropertyData']['legal_document'])){
            return response()->json(['error' => 'Legal document not found.'], 404);
        }

        $filePath = storage_path('app' . '/' . $request->baseDirPath . '/' . $request->file);

        if (!file_exists($filePath)) {
            return response()->json(['error' => 'File not found.'], 404);
        }

        return response()->file($filePath, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="' . $request->file . '"',
        ]);
    }

    public function open_flood_doc(Request $request)
    {
        $order = Order::findOrFail($request->orderid);

        if(!empty($order->input['PropertyData']['legal_document'])){
            return response()->json(['error' => 'Legal document not found.'], 404);
        }

        $filePath = storage_path('app' . '/' . $request->baseDirPath . '/' . $request->file);

        if (!file_exists($filePath)) {
            return response()->json(['error' => 'File not found.'], 404);
        }

        return response()->file($filePath, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="' . $request->file . '"',
        ]);
    }

}
