<?php

namespace App\Http\Controllers;

use DB;
use PDF;
use App\Models\Order;
use App\Models\SupportingDocs;
use mikehaertl\pdftk\Pdf as pdfTk;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Http\Request;
use Image;

class PDFController extends Controller
{
    public function pdfMerger(Request $request)
    {
        $orderId =  $request->orderid;
        $pdfIds =  $request->ids;
        $baseDirPath =  $request->baseDirPath;

        $filePath = storage_path("app/{$baseDirPath}/{$orderId}");
        $pdfFiles = ["{$filePath}/Certificate.pdf"];
        $attachmentDocs = [];
        foreach($pdfIds as $pdfId) {
            $attachmentDocs[] = SupportingDocs::where('id', $pdfId)->value('pdf_file');
        }
        $pdfFiles = array_merge($pdfFiles, array_map(function ($doc) use ($filePath) {
            return "{$filePath}/{$doc}";
        }, $attachmentDocs));

        $gs = (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? resource_path('gs9.55.0/bin/gswin64c.exe') : '/usr/bin/gs';
        $outputFile = "{$filePath}/Merged_Certificate.pdf";
        $command = "{$gs} -sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER -sOutputFile={$outputFile}";

        foreach ($pdfFiles as $file) {
            $command .= ' ' . escapeshellarg($file);
        }

        $descriptorspec = [
            0 => ['pipe', 'r'],
            1 => ['pipe', 'w'],
            2 => ['pipe', 'w'],
        ];

        $process = proc_open($command, $descriptorspec, $pipes);

        $file = SupportingDocs::where('order_id', $orderId)->where('pdf_file', "Merged_Certificate.pdf")->first();

        if (is_resource($process)) {
            fclose($pipes[0]);
            fclose($pipes[1]);
            fclose($pipes[2]);
            while ($status = proc_get_status($process)) {
                if (!$status['running']) {
                    $exitCode = $status['exitcode'];
                    break;
                }
                usleep(100000);
            }
            $endFileCount = count(File::files($filePath));

            $supportingDocid = NULL;
            if($exitCode === 0) {
                if(isset($file)) {
                    $supportingDocid = $file->id;
                } else {
                    $supportingDocid = SupportingDocs::insertGetId(['order_id' => $orderId, 'pdf_file' => "Merged_Certificate.pdf",'created_at' => now()]);
                }
                $response = "Merged PDF Generated Successfully";
            } else {
                $response = "Merged PDF Not Generated";
            }

            proc_close($process);
        } else {
            $response = "Failed to execute the shell command";
        }

        if(substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($filePath, 0777);
        }

        $supportingdocs = SupportingDocs::find($supportingDocid);

        return response()->json($supportingdocs);
    }

    public function changeFolderPermissions($path, $permissions)
    {
        if (file_exists($path) || is_dir($path)) {
            chmod($path, $permissions);
            $items = scandir($path);

            foreach ($items as $item) {
                if ($item != '.' && $item != '..') {
                    $itemPath = $path . '/' . $item;
                    if (is_dir($itemPath)) {
                        $this->changeFolderPermissions($itemPath, $permissions);
                    } else {
                        chmod($itemPath, $permissions);
                    }
                }
            }
        }
    }

    protected function compressAndConvertImageToPdf($file)
    {
        $extension = strtolower($file->getClientOriginalExtension());

        if (!in_array($extension, ['jpg', 'jpeg', 'png'])) {
            return null; // Not a supported image format
        }

        $maxFileSizeForCompression = 1 * 1024 * 1024; // 2MB in bytes

        // Check if the file size exceeds the compression threshold
        if ($file->getSize() > $maxFileSizeForCompression) {
            // Compress and resize the image
            $compressedImage = Image::make($file)->encode('jpg', 80);
        } else {
            // Use the original image without compression
            $compressedImage = Image::make($file);
        }

        $base64Image = 'data:image/jpg;base64,' . base64_encode(file_get_contents($file->path()));

        $pdf = PDF::loadHTML('<img src="' . $base64Image . '" style="width:100%;">')->setPaper('a4');

        return $pdf;
    }

    public function upload_supporting_doc($orderId, $files, $baseDirPath)
    {
        $order = Order::find($orderId);

        $supportingdocids = [];
        $filePath = storage_path("app/{$baseDirPath}/{$order->id}");

        for ($i = 0; $i < count($files); $i++) {
            $file = $files[$i];
            $fileName = $file->getClientOriginalName();

            if ($order->supportingDocs()->where('pdf_file', $fileName)->count() > 0) {
                return response()->json(['message' => 'File already exists with the same name.']);
            }
            $directoryPath = "{$baseDirPath}/{$order->id}";
            $pdf = $this->compressAndConvertImageToPdf($file);

            if (!Storage::exists($directoryPath)) {
                Storage::makeDirectory($directoryPath, 0777, true);
            } else {
                chmod(storage_path("app/{$directoryPath}"), 0777);
            }

            if ($pdf) {
                $pdfFileName = pathinfo($fileName, PATHINFO_FILENAME) . '.pdf';
                $pdfFilePath = "{$directoryPath}/{$pdfFileName}";

                $pdf->save(storage_path("app/{$pdfFilePath}"));

                // Store supporting doc entry in the database
                $supportingDocId = SupportingDocs::insertGetId([
                    'order_id' => $orderId,
                    'pdf_file' => $pdfFileName,
                    'created_at' => now()
                ]);

                $supportingdocids[] = $supportingDocId;
            } else {
                $startFileCount = count(File::files($filePath));
                $file->storeAs($directoryPath, $fileName);
                $endFileCount = count(File::files($filePath));
                $supportingDocid = Null;
                if($startFileCount + 1 == $endFileCount) {
                    $supportingDocid = SupportingDocs::insertGetId(['order_id' => $orderId, 'pdf_file' => $fileName,'created_at' => now()]);
                }
                if(!empty($supportingDocid)){
                    $supportingdocids[] = $supportingDocid;
                }
            }
        }

        $supportingdocs = SupportingDocs::find($supportingdocids);

        if (substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($filePath, 0777);
        }

        return $supportingdocs;
    }

    public function delete_supporting_doc($orderId, $baseDirPath, $fileId)
    {
        $supportingDoc = SupportingDocs::find($fileId);

        if ($supportingDoc) {
            $filePath = "$baseDirPath/{$supportingDoc->order_id}/{$supportingDoc->pdf_file}";
            if (Storage::exists($filePath)) {
                Storage::delete($filePath);
            }
            SupportingDocs::find($fileId)->delete();
            return response()->json(['message' => 'File deleted successfully.']);
        }
        $supportingdocs = SupportingDocs::with('order')->where('order_id', $orderId)->get();
        return response()->json($supportingdocs);
    }

    public function  open_supporting_doc(Request $request)
    {

        $supportingdocs = SupportingDocs::with('order')->find($request->docid);

        $filePath = 'storage/'.$supportingdocs->pdf_path."/".$supportingdocs->pdf_file;

        return Response::file($filePath, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="'. $supportingdocs->pdf_file .'"'
        ]);
    }

}
