<?php

use App\Http\Controllers\HomeController;
use App\Http\Controllers\SettingController;
use App\Http\Controllers\AssignOrderController;
use App\Http\Controllers\OrderCreationController;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\PDFController;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\FloodCertPDFController;
use App\Http\Controllers\PassportController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "web" middleware group. Make something great!
|
*/

Auth::routes();

Route::middleware('auth:web')->controller(HomeController::class)->group(function () {
    Route::get('/', 'index')->name('home');
    Route::get('home', 'index')->name('home');

    Route::get('profile_Edit', 'profileEdit')->name('profile_Edit');
    Route::post('profile_Update', 'profileupdate')->name('profile_Update');
    Route::post('api_token', 'apitoken')->name('api_token');
    Route::get('/pdf/{folder}/{filename}', 'showPdf')->name('pdf.show');
});

Route::middleware(['auth:web', 'role_or:Admin,Super Admin'])->controller(SettingController::class)->group(function () {
    Route::get('settings', 'setting')->name('settings');
    Route::get('settings/users', 'setting')->name('users');
    Route::get('settings/services', 'setting')->name('services');
    Route::get('settings/status', 'setting')->name('status');



    Route::post('getCounty', 'getCounty')->name('getCounty');
    Route::post('getClients', 'getClients')->name('getClients');
    // Users
    Route::post('/usersInsert', 'addUsers')->name('usersInsert');
    Route::post('edit_user', 'edit_user')->name('edit_user');
    Route::post('/updateUsers', 'updateUsers')->name('updateUsers');
    Route::any('/userStatus/{userid?}', 'userStatus');
    Route::post('/assignmentData', 'assignmentData')->name('assignmentData');
    Route::any('/assignService', 'assignService')->name('assignService');
    Route::post('/removeassignService', 'removeassignService')->name('removeassignService');
    Route::post('/show_user', 'showUser')->name('show_user');
    // Ajax Post
    Route::post('getCounty', 'getCounty')->name('getCounty');
    // Services
    Route::post('/serviceInsert', 'addservice')->name('serviceInsert');
    Route::post('edit_service', 'edit_service')->name('edit_service');
    Route::post('/updateService', 'updateService')->name('updateService');
    Route::any('/serviceStatus/{service?}', 'serviceStatus');

});

Route::middleware(['auth:web', 'role_or:Super Admin'])->controller(SettingController::class)->group(function () {
    Route::get('settings/company', 'setting')->name('company');
    Route::get('settings/client', 'setting')->name('client');
    Route::any('settings/order_delete', 'setting')->name('order_delete');
    // Company
    Route::post('/companyInsert', 'addCompany')->name('companyInsert');
    Route::post('edit_company', 'edit_company')->name('edit_company');
    Route::post('/updateCompany', 'updateCompany')->name('updateCompany');
    Route::any('/companyStatus/{company?}', 'companyStatus');
    // Client
    Route::post('/clientInsert', 'addClient')->name('clientInsert');
    Route::post('edit_client', 'edit_client')->name('edit_client');
    Route::post('/updateClient', 'updateClient')->name('updateClient');
    Route::any('/clientStatus/{client?}', 'clientStatus');

    // Order Delete
    Route::post('/updateOrderStatus', 'updateOrderStatus')->name('update_order_status');

});

Route::middleware(['auth:web', 'role_or:Lead,Admin,Super Admin'])->controller(AssignOrderController::class)->group(function () {
    Route::get('ordersAssignment', 'ordersAssignment')->name('ordersAssignment');
    Route::get('ordersAssignment/assign_orders', 'ordersAssignment')->name('assign_orders');
    Route::get('ordersAssignment/assigned_orders', 'ordersAssignment')->name('assigned_orders');
    Route::get('ordersAssignment/unassigned_orders', 'ordersAssignment')->name('unassigned_orders');
    Route::post('assignment_update', 'assignment_update')->name('assignment_update');

    Route::post('getServiceUsers', 'getServiceUsers')->name('getServiceUsers');
});

Route::middleware('auth:web')->controller(OrderCreationController::class)->group(function () {
    Route::get('single_order', 'single_order')->name('single_order');
    Route::post('InsertOrder', 'InsertOrder')->name('InsertOrder');
});

Route::middleware('auth:web')->controller(OrderController::class)->group(function () {
    Route::get('orders', 'orders')->name('orders');
    Route::post('getOrderData', 'getOrderData')->name('getOrderData');
    Route::get('orders/{order_id}', 'orderform')->name('orderform');
    Route::post('/assignOrder', 'assignOrder')->name('assignOrder');

    Route::post('submitOrderData', 'submitOrderData')->name('submitOrderData');
    Route::post('open_supporting_doc', 'open_supporting_doc')->name('open_supporting_doc');
    Route::any('upload_supporting_doc', 'upload_supporting_doc')->name('upload_supporting_doc');
    Route::post('delete_supporting_doc', 'delete_supporting_doc')->name('delete_supporting_doc');

    Route::post('submitLegalVestingData', 'submitLegalVestingData')->name('submitLegalVestingData');

    Route::post('order_status_history', 'order_status_history')->name('order_status_history');
    Route::post('other_status_update', 'other_status_update')->name('other_status_update');
    Route::post('order_status_change', 'order_status_change')->name('order_status_change');

    Route::post('getAttachments', 'getAttachments')->name('getAttachments');
    Route::post('/open_legal_doc', 'open_legal_doc')->name('open_legal_doc');
    Route::post('/open_flood_doc', 'open_flood_doc')->name('open_flood_doc');
});


Route::middleware('auth:web')->controller(PDFController::class)->group(function () {
    Route::post('pdfMerger', 'pdfMerger')->name('pdfMerger');

});
